/********************************************************************************
 *  Portions Copyright (C) 2000-2003  Enterprise Distributed Technologies Ltd
 *
 *  This file is dual licensed under the terms of the Apache Software License
 *  Version 2.0, and the GNU Lesser GPL Version 2.1 as provided for by the
 *  edtFTPj project (http://www.enterprisedt.com/products/edtftpj/readme.html#Licensing).
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *******************************************************************************/

package org.apache.hop.workflow.actions.ftpput;

import java.io.IOException;
import java.text.ParseException;

import org.apache.hop.core.logging.ILogChannel;
import org.apache.hop.i18n.BaseMessages;

import com.enterprisedt.net.ftp.FTPClient;
import com.enterprisedt.net.ftp.FTPException;
import com.enterprisedt.net.ftp.FTPFile;

/**
 * This class should largely be able to be removed if the edtftp project accepts my change to replace dirDetails(".")
 * with dirDetails(null).
 *
 * @author mbatchelor
 */

public class FtpClient extends FTPClient {

  /**
   * MDTM supported flag
   */
  private boolean mdtmSupported = true;

  /**
   * SIZE supported flag
   */
  private boolean sizeSupported = true;

  private static final Class<?> PKG = FtpClient.class; // For Translator
  private ILogChannel log;

  public FtpClient( ILogChannel log ) {
    super();
    this.log = log;
    log.logBasic( BaseMessages.getString( PKG, "PDIFTPClient.DEBUG.Using.Overridden.FTPClient" ) );
  }

  /*
   * (non-Javadoc)
   *
   * @see com.enterprisedt.net.ftp.FTPClientInterface#exists(java.lang.String)
   */
  public boolean exists( String remoteFile ) throws IOException, FTPException {
    checkConnection( true );

    // first try the SIZE command
    if ( sizeSupported ) {
      lastReply = control.sendCommand( "SIZE " + remoteFile );
      char ch = lastReply.getReplyCode().charAt( 0 );
      if ( ch == '2' ) {
        return true;
      }
      if ( ch == '5' && fileNotFoundStrings.matches( lastReply.getReplyText() ) ) {
        return false;
      }

      sizeSupported = false;
      log.logDebug( "SIZE not supported - trying MDTM" );
    }

    // then try the MDTM command
    if ( mdtmSupported ) {
      lastReply = control.sendCommand( "MDTM " + remoteFile );
      char ch = lastReply.getReplyCode().charAt( 0 );
      if ( ch == '2' ) {
        return true;
      }
      if ( ch == '5' && fileNotFoundStrings.matches( lastReply.getReplyText() ) ) {
        return false;
      }

      mdtmSupported = false;
      log.logDebug( "MDTM not supported - trying LIST" );
    }

    try {
      FTPFile[] files = dirDetails( null ); // My fix - replace "." with null in this call for MVS support
      for ( int i = 0; i < files.length; i++ ) {
        if ( files[ i ].getName().equals( remoteFile ) ) {
          return files[ i ].isFile();
        }
      }
      return false;
    } catch ( ParseException ex ) {
      log.logBasic( ex.getMessage() );
      return false;
    }
  }

}
